--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'buff'

local Addon    = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L        = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')
local AceGUI   = LibStub( 'AceGUI-3.0')

local MAX_BUTTON  = 40
local FILTER      = 'HELPFUL'

local Module = Addon:NewBarModule( MAJOR, 'AceHook-3.0')
Module.filter  = FILTER
Module.color   = { r=0.2, g=0.8, b=0.2 }

local options = {
	type = 'group', order = 20, name = L.BarBuff, handler = Module, dialogHidden = true, dialogInline = true, get = 'GetProperty', set = 'SetProperty', 
	args = {
		descr  = { type = 'description', order = 5, name = L.DescBuff, cmdHidden = true, fontSize = 'large' },
		option = { 
			type = 'group', order = 10, name = L.OptionName, inline = true, 
			args = {
				show     = { type = 'toggle', order = 10, name = L.ShowName,     desc = L.ShowDesc },
				join     = { type = 'toggle', order = 15, name = L.JoinName,     desc = L.JoinDesc,     set = 'SetJoin',     get = 'GetJoin' },
				flashing = { type = 'toggle', order = 20, name = L.FlashingName, desc = L.FlashingDesc, set = 'SetFlashing' },
				timer    = { type = 'toggle', order = 30, name = L.TimerName,    desc = L.TimerDesc },
				sort     = { type = 'select', order = 40, name = L.SortName,     desc = L.SortDesc,     set = 'SetSortType', get = 'GetSortType', values = 'GetSortDesc' },
				spell    = { type = 'toggle', order = 50, name = L.SpellName,    desc = L.SpellDesc },
			} 
		},
		layout = { 
			type = 'group', order = 20, name = L.BarName, inline = true, 
			args = {
				horizontal = { type = 'toggle', order = 10, name = L.HorizontalName, desc = L.HorizontalDesc },
				number     = { type = 'range',  order = 20, name = L.NumberName,     desc = L.NumberDesc,   set = 'SetNumber',   min = 1,    max = MAX_BUTTON, step = 1 },
				gap        = { type = 'range',  order = 30, name = L.GapName,        desc = L.GapDesc,      set = 'SetGap',      min = -20,  max = 20,         step = 1 },
				scale      = { type = 'range',  order = 40, name = L.ScaleName,      desc = L.ScaleDesc,    set = 'SetScale',    min = 0.01, max = 2,          step = 0.01, isPercent = true },
				cols       = { type = 'range',  order = 50, name = L.ColsName,       desc = L.ColsDesc,     set = 'SetCols',     min = 1,    max = MAX_BUTTON, step = 1 },
				xPadding   = { type = 'range',  order = 60, name = L.XPaddingName,   desc = L.XPaddingDesc, set = 'SetXPadding', min = -20,  max = 20,         step = 1 },
				rows       = { type = 'range',  order = 70, name = L.RowsName,       desc = L.RowsDesc,     set = 'SetRows',     min = 1,    max = MAX_BUTTON, step = 1 },
				yPadding   = { type = 'range',  order = 80, name = L.YPaddingName,   desc = L.YPaddingDesc, set = 'SetYPadding', min = -50,  max = 50,         step = 1 },
			} 
		}
	},
}

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function Button_OnEnter( btn)
	local id = btn:GetUserData( 'id')
	if id then
		GameTooltip:SetOwner( btn.frame, 'ANCHOR_BOTTOMLEFT')
		GameTooltip:SetUnitAura( PlayerFrame.unit, id, FILTER)
	else
		local hidden = Addon:GetModule( 'hidden', true)
		if hidden then
			hidden.group:ClearAllPoints()
			hidden.group:SetPoint( 'TOPRIGHT', btn.frame, 'BOTTOMLEFT', 0, 0)
			hidden:SetVisible( true)
		else
			Module:Print( 'missing module hidden')
		end
	end
end

local function Button_OnLeave( btn)
	if btn:GetUserData( 'id') then
		GameTooltip:Hide()
	else
		local hidden = Addon:GetModule( 'hidden', true)
		if hidden then
			hidden:SetVisible( false)
		else
			Module:Print( 'missing module hidden')
		end
	end
end

local function Button_OnClick( btn)
	StaticPopupDialogs['Bison_Cancel_Aura'] = StaticPopupDialogs['Bison_Cancel_Aura'] or {
		text = L.CancelAura,
		button2 = ACCEPT,
		hasEditBox = 1,
		hasWideEditBox = 1,
		OnShow = function( self, ...)
			self:SetWidth(450)
			local editBox = _G[self:GetName()..'WideEditBox'] or _G[self:GetName()..'EditBox']
		    editBox:SetText( '/cancelaura '..StaticPopupDialogs['Bison_Cancel_Aura'].buffName)
		    editBox:SetFocus()
		    editBox:HighlightText(0)
		    editBox:SetWidth( 250)
		    local button = _G[self:GetName()..'Button2']
		    button:ClearAllPoints()
		    button:SetWidth( 200)
		    button:SetPoint( 'CENTER', editBox, 'CENTER', 0, -30)
		end,
		OnHide = function() end,
		OnAccept = function() end,
		OnCancel = function() end,
		EditBoxOnEscapePressed = function( self, ...)
			self:GetParent():Hide()
		end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}
	local name = btn:GetUserData( 'name')
	StaticPopupDialogs['Bison_Cancel_Aura'].buffName = name
	StaticPopup_Show( 'Bison_Cancel_Aura', name)
end

local function Button_OnMouseDown( btn)
	btn.parent:Fire( 'OnMouseDown')
end

local function Button_OnMouseUp( btn)
	btn.parent:Fire( 'OnMouseUp')
end

local function Button_OnTooltip( btn)
	local id = btn:GetUserData( 'id')
	if id and GameTooltip:IsOwned( btn.frame) then
		GameTooltip:SetUnitAura( PlayerFrame.unit, id, FILTER)
	end
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Module:OnModuleInitialize()
	self:RegisterOptions( options)
	self:CloneAura( 'buff')
	self:RawHook( self, 'GetSort', 'GetSortConsolidation')
end

function Module:CreateButton()
	local btn = AceGUI:Create( 'BisonBuff')
	btn:SetCallback( 'OnEnter', Button_OnEnter)
	btn:SetCallback( 'OnLeave', Button_OnLeave)
	btn:SetCallback( 'OnClick', Button_OnClick)
	btn:SetCallback( 'OnMouseDown', Button_OnMouseDown)
	btn:SetCallback( 'OnMouseUp',   Button_OnMouseUp)
	btn:SetCallback( 'OnTooltip',   Button_OnTooltip)
	return btn
end

function Module:UpdateValues()
	local cons = self:GetConsolidation()
	local lock = self:IsLock()
	for i,child in pairs( self.group.children) do
		local a = nil
		if cons < 2 then
			a = self.aura[i]
		elseif i > 1 then
			a = self.aura[i - 1]
		end
		if a then
			self:UpdateChild( child, a)
			child:SetLock( lock)
			if cons > 1 and a.hide then
				child:SetVisible( false)
			else
				child:SetVisible( a.name or not lock)
			end
		else
			child:SetUserData( 'id', nil)
			child:SetText( cons)
			child:SetCount( nil)
			child:SetSymbol( nil)
			child:SetTexture( 'Interface\\Buttons\\BuffConsolidation', 0.125, 0.375, 0.25, 0.75)
			child:SetBorder( nil)
			child:SetLock( lock)
			child:SetVisible( true)
		end
	end
end

function Module:GetSortConsolidation( self)
	local cons = self:GetConsolidation()
	local org = self.hooks[self]['GetSort']( self)
	return function( a, b) 
		if cons > 1 then
			if a.hide then return false end
			if b.hide then return true end
		end
		return org( a, b)
	end
end
