--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'hidden'

local Addon    = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L        = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')
local AceGUI   = LibStub( 'AceGUI-3.0')

local PANE_BACKDROP  = {
	bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background', tile = true, tileSize = 16,
	edgeFile = 'Interface\\Tooltips\\UI-Tooltip-Border', edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 },
}
local MAX_BUTTON  = 32
local FILTER      = 'HELPFUL'

local Module = Addon:NewBarModule( MAJOR, 'AceHook-3.0')
Module.filter  = FILTER
Module.color   = { r=0.4, g=0.4, b=0.2 }
Module.hideOnLock = true

local options = {
	type = 'group', order = 30, name = L.BarHidden, handler = Module, dialogHidden = true, dialogInline = true, get = 'GetProperty', set = 'SetProperty', 
	args = {
		descr  = { type = 'description', order = 5, name = L.DescHidden, cmdHidden = true, fontSize = 'large' },
		option = { 
			type = 'group', order = 10, name = L.OptionName, inline = true, 
			args = {
				show     = { type = 'toggle', order = 10, name = L.ShowName,     desc = L.ShowDesc },
				timer    = { type = 'toggle', order = 30, name = L.TimerName,    desc = L.TimerDesc },
				sort     = { type = 'select', order = 40, name = L.SortName,     desc = L.SortDesc, get = 'GetSortType', set = 'SetSortType', values = 'GetSortDesc' },
			} 
		},
		layout = { 
			type = 'group', order = 20, name = L.BarName, inline = true, 
			args = {
				horizontal = { type = 'toggle', order = 10, name = L.HorizontalName, desc = L.HorizontalDesc },
				number     = { type = 'range',  order = 20, name = L.NumberName,     desc = L.NumberDesc,   set = 'SetNumber',   min = 1,    max = MAX_BUTTON, step = 1 },
				gap        = { type = 'range',  order = 30, name = L.GapName,        desc = L.GapDesc,      set = 'SetGap',      min = -20,  max = 20,         step = 1 },
				scale      = { type = 'range',  order = 40, name = L.ScaleName,      desc = L.ScaleDesc,    set = 'SetScale',    min = 0.01, max = 2,          step = 0.01, isPercent = true },
				cols       = { type = 'range',  order = 50, name = L.ColsName,       desc = L.ColsDesc,     set = 'SetCols',     min = 1,    max = MAX_BUTTON, step = 1 },
				xPadding   = { type = 'range',  order = 60, name = L.XPaddingName,   desc = L.XPaddingDesc, set = 'SetXPadding', min = -20,  max = 20,         step = 1 },
				rows       = { type = 'range',  order = 70, name = L.RowsName,       desc = L.RowsDesc,     set = 'SetRows',     min = 1,    max = MAX_BUTTON, step = 1 },
				yPadding   = { type = 'range',  order = 80, name = L.YPaddingName,   desc = L.YPaddingDesc, set = 'SetYPadding', min = -50,  max = 50,         step = 1 },
			} 
		}
	},
}

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function Button_OnMouseDown( btn)
	btn.parent:Fire( 'OnMouseDown')
end

local function Button_OnMouseUp( btn)
	btn.parent:Fire( 'OnMouseUp')
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Module:OnModuleInitialize()
	self:RegisterOptions( options)
	self:CloneAura( 'buff')
	self:RawHook( self, 'GetSort', 'GetSortConsolidation')
	self.group:SetBackdrop( PANE_BACKDROP)
--	self.group:SetBackdropColor( TOOLTIP_DEFAULT_BACKGROUND_COLOR)
end

function Module:OnModuleEnable()
	self:SetVisible( false)
end

function Module:CreateButton()
	local btn = AceGUI:Create( 'BisonBuff')
	btn:SetCallback( 'OnMouseDown', Button_OnMouseDown)
	btn:SetCallback( 'OnMouseUp',   Button_OnMouseUp)
	return btn
end

function Module:UpdateValues()
	local cons = self:GetConsolidation()
	local lock = self:IsLock()
	for i,child in pairs( self.group.children) do
		local a = self.aura[i]
		self:UpdateChild( child, a)
		child:SetLock( lock)
		if not lock then
			child:SetVisible( true)
		elseif cons > 0 and a.hide then
			child:SetVisible( a.name)
		else
			child:SetVisible( false)
		end
	end
end

function Module:GetSortConsolidation( self)
	local org = self.hooks[self]['GetSort']( self)
	return function( a, b) 
		if a.hide and b.hide then return org( a, b) end
		if a.hide then return true end
		if b.hide then return false end
		return a.id < b.id
	end
end
