--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'weapon'

local Addon  = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L      = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')
local AceGUI = LibStub( 'AceGUI-3.0')

local UPDATE_TIME = 0.2
local MAX_BUTTON  = 2

local Module = Addon:NewBarModule( MAJOR)
Module.color   = { r=0.2, g=0.2, b=0.2 }

local options = {
	type = 'group', order = 40, name = L.BarWeapon, handler = Module, dialogHidden = true, dialogInline = true, get = 'GetProperty', set = 'SetProperty', 
	args = {
		descr  = { type = 'description', order = 5, name = L.DescWeapon, cmdHidden = true, fontSize = 'large' },
		option = { 
			type = 'group', order = 10, name = L.OptionName, inline = true, 
			args = {
				show     = { type = 'toggle', order = 10, name = L.ShowName,     desc = L.ShowDesc, width = 'full' },
				flashing = { type = 'toggle', order = 20, name = L.FlashingName, desc = L.FlashingDesc },
				timer    = { type = 'toggle', order = 30, name = L.TimerName,    desc = L.TimerDesc },
				sort     = { type = 'select', order = 40, name = L.SortName,     desc = L.SortDesc, get = 'GetSortType', set = 'SetSortType', values = 'GetSortDesc' },
			} 
		},
		layout = { 
			type = 'group', order = 20, name = L.BarName, inline = true, 
			args = {
				horizontal = { type = 'toggle', order = 10, name = L.HorizontalName, desc = L.HorizontalDesc, width = 'full' },
				gap        = { type = 'range',  order = 30, name = L.GapName,        desc = L.GapDesc,      set = 'SetGap',      min = -20,  max = 20,         step = 1 },
				scale      = { type = 'range',  order = 40, name = L.ScaleName,      desc = L.ScaleDesc,    set = 'SetScale',    min = 0.01, max = 2,          step = 0.01, isPercent = true },
				cols       = { type = 'range',  order = 50, name = L.ColsName,       desc = L.ColsDesc,     set = 'SetCols',     min = 1,    max = MAX_BUTTON, step = 1 },
				xPadding   = { type = 'range',  order = 60, name = L.XPaddingName,   desc = L.XPaddingDesc, set = 'SetXPadding', min = -20,  max = 20,         step = 1 },
				rows       = { type = 'range',  order = 70, name = L.RowsName,       desc = L.RowsDesc,     set = 'SetRows',     min = 1,    max = MAX_BUTTON, step = 1 },
				yPadding   = { type = 'range',  order = 80, name = L.YPaddingName,   desc = L.YPaddingDesc, set = 'SetYPadding', min = -50,  max = 50,         step = 1 },
			} 
		}
	},
}

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function Button_OnEnter( btn)
	local id = btn:GetUserData( 'id')
	GameTooltip:SetOwner( btn.frame, 'ANCHOR_BOTTOMLEFT')
	GameTooltip:SetInventoryItem( PlayerFrame.unit, id)
end

local function Button_OnLeave( btn)
	if GameTooltip:IsOwned( btn.frame) then
		GameTooltip:Hide()
	end
end

local function Button_OnMouseDown( btn)
	btn.parent:Fire( 'OnMouseDown')
end

local function Button_OnMouseUp( btn)
	btn.parent:Fire( 'OnMouseUp')
end

local function Button_OnTooltip( btn)
	if GameTooltip:IsOwned( btn.frame) then
		local id = btn:GetUserData( 'id')
		GameTooltip:SetInventoryItem( PlayerFrame.unit, id)
	end
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Module:OnModuleInitialize()
	self:RegisterOptions( options)
	self:CloneAura( 'weapon')
end

function Module:CreateButton()
	local btn = AceGUI:Create( 'BisonBuff')
	btn:SetCallback( 'OnEnter', Button_OnEnter)
	btn:SetCallback( 'OnLeave', Button_OnLeave)
	btn:SetCallback( 'OnMouseDown', Button_OnMouseDown)
	btn:SetCallback( 'OnMouseUp',   Button_OnMouseUp)
	btn:SetCallback( 'OnTooltip',   Button_OnTooltip)
	return btn
end

function Module:UpdateValues()
	local lock = self:IsLock()
	for i,child in pairs( self.group.children) do
		local a = self.aura[i]
		self:UpdateChild( child, a)
		child:SetLock( lock)
		child:SetVisible( a.name or not lock)
	end
end
